<?php

/**
 * Timetable.class.php
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * @author Heiko Dostert <dosterth@uni-trier.de>
 * @package portaLE
 * @copyright Universitt Trier, Koordinationsstelle E-Learning, 2010
 * @version 1.0.2 - 22.11.2010 - Jrg Rpke <roepke@uni-trier.de>
 */

require_once dirname(__FILE__) . '/OmGateway.class.php';
require_once dirname(__FILE__) . '/Classroom.class.php';
require_once dirname(__FILE__) . '/PortaleException.class.php';
require_once dirname(__FILE__) . '/config_portale.inc.php';

class Timetable
{
	static function cleanPortaLE()
	{	
		if(CLEAN == 0)
			return;
			
		$db = DBManager::Get();
		$sql = "SELECT date FROM portaLE_services WHERE type='clean'";
		$result = $db->query($sql)->fetch();
		if(($result['date'] + CLEAN) < time()) {
			// Timestamp aktualisieren
			$sql = "REPLACE INTO portaLE_services SET type='clean', date='".time()."'";
			$db->exec($sql);
			// portaLE_conferences aufrumen
			$sql = "DELETE FROM portaLE_conferences USING seminare, portaLE_conferences WHERE (SELECT seminare.Seminar_id FROM seminare WHERE portaLE_conferences.seminar_id = seminare.Seminar_id) IS NULL";
			$db->exec($sql);
			// portaLE_waitingroom aufrumen
			$sql = "DELETE FROM portaLE_waitingroom USING portaLE_conferences, portaLE_waitingroom WHERE (SELECT portaLE_conferences.id FROM portaLE_conferences WHERE portaLE_conferences.id = portaLE_waitingroom.conference_id) IS NULL";
			$db->exec($sql);
		}
	}
	
	//Read the list of classrooms from the db:
	static function getRooms($teacherPermission=true)
	{
		global $OM_SERVERS;
		$rooms = array();
		$db = DBManager::Get();
		if($teacherPermission)  //teacher see also past conferences
			$q = sprintf("SELECT portaLE_conferences.*, auth_user_md5.Vorname, auth_user_md5.Nachname FROM portaLE_conferences, auth_user_md5 WHERE portaLE_conferences.teacher = auth_user_md5.user_id AND portaLE_conferences.seminar_id = %s ORDER BY date", $db->quote($_SESSION['SessSemName'][1]));
		else  //students don't see past conferences
			$q = sprintf("SELECT portaLE_conferences.*, auth_user_md5.Vorname, auth_user_md5.Nachname FROM portaLE_conferences, auth_user_md5 WHERE portaLE_conferences.teacher = auth_user_md5.user_id AND portaLE_conferences.seminar_id = %s AND (DATE_ADD(portaLE_conferences.date, INTERVAL portaLE_conferences.length SECOND)> now()) ORDER BY date", $db->quote($_SESSION['SessSemName'][1]));
		$results = $db->query($q)->fetchAll(PDO::FETCH_ASSOC);
		foreach ($results as $result)
		{
			$date = strtotime($result['date']);
//			if(($date+$result['length'])<time()){  //if conference is finished, ask for recordings
				$server = $OM_SERVERS[$result['server']];
				$options = array("location" => $server['SHARINGINTERFACE_LOCATION'],       
						"uri" => $server['SHARINGINTERFACE_URI'],                  
						"style" => $server['SHARINGINTERFACE_STYLE'],                                
						"use" => $server['SHARINGINTERFACE_USE']);
				try{
					//print_r($OM_SERVERS);
					$om_interface = new SoapClient(null, $options);
					
					$recordings = $om_interface->getRefRecords($result['rooms_id'], $server['SEC_KEY']);
				}
				catch(Exception $e){
					throw new PortaleException(_("Beim Zugriff auf den Server ist ein Fehler aufegtreten. Bitte wenden Sie sich an den Administrator."));
				}
//			}
			include("timeline.inc.php");
			array_push($rooms, new Classroom($result['id'], $result['title'], $result['server'], $result['rooms_id'], $result['roomtype'], $result['Vorname'].' '.$result['Nachname'], $date, $result['length'], $result['numberOfParticipants'], $recordings, $timeline, ($result['waitingroomClosed']==1)));
		}
		return $rooms;
	}


	//Create a new room:
  	static function addRoom($title, $roomtype, $teacher, $hour, $minute, $month, $day, $year, $length, $numberOfParticipants)
	{
		global $OM_SERVERS, $ROOMTYPES;
		$db = DBManager::Get();
		if(!checkdate($month, $day, $year))
			throw new PortaleException(_("Bitte berprfen Sie die Datumseingabe"));
		$date = mktime($hour, $minute, '0', $month, $day, $year);
		if($date < time())
			throw new PortaleException(_("Der angegebene Termin liegt in der Vergangenheit"));
		if((RESERVATION_PERIOD > 0) && ($date > (time() + (RESERVATION_PERIOD * 86400))))
			throw new PortaleException(_("Virtuelle Rume knnen nur ").RESERVATION_PERIOD._(" Tage im Voraus reserviert werden"));
		if(MAX_CONFERENCE_NUMBER > 0){
			$q = sprintf("SELECT count(*) AS number FROM portaLE_conferences WHERE seminar_id = %s GROUP BY seminar_id", $db->quote($_SESSION['SessSemName'][1]));
			$result = $db->query($q)->fetchAll(PDO::FETCH_ASSOC);
			if($result['number'] >= MAX_CONFERENCE_NUMBER)
				throw new PortaleException(_("Es sind zur Zeit ".$result['number']." Rume reserviert. Es knnen maximal ").MAX_CONFERENCE_NUMBER._(" virtuelle Rume fr eine Veranstaltung reserviert werden. Bitte lschen Sie einen Raum, bevor Sie einen neuen anlegen."));
		}

		//Search an om server with enough free capacity:
		$maxCapacity = 0;
		$mysqlstartdate = date( 'Y-m-d H:i:s', $date);
		$mysqlenddate = date( 'Y-m-d H:i:s', ($date + $length));
		foreach($OM_SERVERS AS $key => $value){
			$options = array("location" => $value['SHARINGINTERFACE_LOCATION'],       
						"uri" => $value['SHARINGINTERFACE_URI'],                  
						"style" => $value['SHARINGINTERFACE_STYLE'],                                
						"use" => $value['SHARINGINTERFACE_USE']);
			try{
				$interface = new SoapClient(null, $options);
				$capacity = $interface->getCapacity($mysqlstartdate, $mysqlenddate, $value['SEC_KEY']);
			}
			catch(Exception $e){
				if(defined('MAIL_ADMIN'))
					mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), sprintf(_('Beim Zugriff auf das SharingInterface ist in der Funktion addRoom (Zeile %d, Datei Timetable.class.php) der folgende Fehler aufgetreten: %s'), __LINE__, $e));
				throw new PortaleException(_("Beim Zugriff auf den Server ist ein Fehler aufegtreten. Bitte wenden Sie sich an den Administrator."));
					
			}
			if($capacity === false){
				if(defined('MAIL_ADMIN'))
					mail(MAIL_ADMIN, _('Fehler bei Openmeetings-Zugriff'), sprintf(_('Beim Zugriff auf das SharingInterface ist in der Funktion addRoom (Zeile %d, Datei Timetable.class.php) der folgende Fehler aufgetreten: Falsches Passwort fr den Zugriff auf das Interface.'), __LINE__));
				throw new PortaleException(_("Beim Zugriff auf den Server ist ein Fehler aufgetreten. Bitte wenden Sie sich an den Administrator."));
			}
			if($capacity > $maxCapacity){
				$maxCapacity = $capacity;
				$om_server = $key;
			}
			if($maxCapacity >= $numberOfParticipants)
				break;
		}
		if($maxCapacity == 0)
			throw new PortaleException(_("Aufgrund einer hohen Server-Auslastung knnen fr den gewnschten Zeitraum leider keine Reservierungen mehr angenommen werden."));
		if($maxCapacity < $numberOfParticipants)
			throw new PortaleException(_("Aufgrund einer hohen Server-Auslastung steht zum gewnschten Zeitpunkt nur noch eine Kapazitt fr maximal $maxCapacity TeilnehmerInnen zur Verfgung. Falls Ihnen diese Raumgre gengt, passen Sie Ihre Einstellung bitte entsprechend an und wiederholen den Reservierungsvorgang."));
		$server = new OmGateway($om_server);
		$room_id = $server->createRoomWithMod($title, $ROOMTYPES[$roomtype], $numberOfParticipants, '', $date, $length);
		$db->exec("INSERT INTO portaLE_conferences SET seminar_id = ".$db->quote($_SESSION['SessSemName'][1]).", server = ".$db->quote($om_server).", rooms_id = $room_id, title = ".$db->quote($title).", teacher = ".$db->quote($teacher).", date = ".$db->quote($mysqlstartdate).", length = $length, roomtype = ".$db->quote($roomtype).", numberOfParticipants = $numberOfParticipants, creationTime=Now()");
	}

	
	//Delete a room:
  	static function deleteRoom($conference_id)
	{
		if(!is_int($conference_id))
			return false;
		$classroom = Timetable::getRoomById($conference_id);
		if($classroom === false)
			return false;
		if($classroom->seminar_id != $_SESSION['SessSemName'][1])
			return false;
		$server = new OmGateway($classroom->server);
		$server->deleteRoom($classroom->rooms_id);
		$db = DBManager::get();
		$db->exec("DELETE FROM portaLE_conferences WHERE id = $conference_id");
		$db->exec("DELETE FROM portaLE_waitingroom WHERE conference_id = $conference_id");
		return true;
	}
	
	
	static function getNumberOfNewRooms($seminar_id, $lastlogin){
		$date = date( 'Y-m-d H:i:s', $lastlogin);
		$db = DBManager::Get();
		$q = sprintf("SELECT count(*) AS number FROM portaLE_conferences WHERE seminar_id = %s AND creationTime > %s", $db->quote($seminar_id), $db->quote($date));
		$data = $db->query($q)->fetch();
		if($data === false)
			return false;
		return $data['number'];
	}
	
	
	static function getRoomById($id=false){
		global $OM_SERVERS;
		if(!is_int($id))
			return false;
		$db = DBManager::get();
		$q = sprintf("SELECT * FROM portaLE_conferences WHERE portaLE_conferences.id = %d AND seminar_id = %s", $id, $db->quote($_SESSION['SessSemName'][1]));
		return $db->query($q)->fetchObject();
	}

}
?>